var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var remote_exports = {};
__export(remote_exports, {
  createRemoteJWKSet: () => createRemoteJWKSet
});
var import_fetch_jwks = __toESM(require("../runtime/fetch_jwks.js"));
var import_import = require("../key/import.js");
var import_errors = require("../util/errors.js");
var import_is_object = __toESM(require("../lib/is_object.js"));
function getKtyFromAlg(alg) {
  switch (typeof alg === "string" && alg.substr(0, 2)) {
    case "RS":
    case "PS":
      return "RSA";
    case "ES":
      return "EC";
    case "Ed":
      return "OKP";
    default:
      throw new import_errors.JOSENotSupported('Unsupported "alg" value for a JSON Web Key Set');
  }
}
function isJWKLike(key) {
  return (0, import_is_object.default)(key);
}
class RemoteJWKSet {
  constructor(url, options) {
    this._cached = /* @__PURE__ */ new WeakMap();
    if (!(url instanceof URL)) {
      throw new TypeError("url must be an instance of URL");
    }
    this._url = new URL(url.href);
    this._options = { agent: options == null ? void 0 : options.agent };
    this._timeoutDuration = typeof (options == null ? void 0 : options.timeoutDuration) === "number" ? options == null ? void 0 : options.timeoutDuration : 5e3;
    this._cooldownDuration = typeof (options == null ? void 0 : options.cooldownDuration) === "number" ? options == null ? void 0 : options.cooldownDuration : 3e4;
  }
  coolingDown() {
    if (!this._cooldownStarted) {
      return false;
    }
    return Date.now() < this._cooldownStarted + this._cooldownDuration;
  }
  async getKey(protectedHeader, token) {
    const joseHeader = __spreadValues(__spreadValues({}, protectedHeader), token.header);
    if (!this._jwks) {
      await this.reload();
    }
    const candidates = this._jwks.keys.filter((jwk2) => {
      let candidate = jwk2.kty === getKtyFromAlg(joseHeader.alg);
      if (candidate && typeof joseHeader.kid === "string") {
        candidate = joseHeader.kid === jwk2.kid;
      }
      if (candidate && typeof jwk2.alg === "string") {
        candidate = joseHeader.alg === jwk2.alg;
      }
      if (candidate && typeof jwk2.use === "string") {
        candidate = jwk2.use === "sig";
      }
      if (candidate && Array.isArray(jwk2.key_ops)) {
        candidate = jwk2.key_ops.includes("verify");
      }
      if (candidate && joseHeader.alg === "EdDSA") {
        candidate = jwk2.crv === "Ed25519" || jwk2.crv === "Ed448";
      }
      if (candidate) {
        switch (joseHeader.alg) {
          case "ES256":
            candidate = jwk2.crv === "P-256";
            break;
          case "ES256K":
            candidate = jwk2.crv === "secp256k1";
            break;
          case "ES384":
            candidate = jwk2.crv === "P-384";
            break;
          case "ES512":
            candidate = jwk2.crv === "P-521";
            break;
          default:
        }
      }
      return candidate;
    });
    const { 0: jwk, length } = candidates;
    if (length === 0) {
      if (this.coolingDown() === false) {
        await this.reload();
        return this.getKey(protectedHeader, token);
      }
      throw new import_errors.JWKSNoMatchingKey();
    } else if (length !== 1) {
      throw new import_errors.JWKSMultipleMatchingKeys();
    }
    const cached = this._cached.get(jwk) || this._cached.set(jwk, {}).get(jwk);
    if (cached[joseHeader.alg] === void 0) {
      const keyObject = await (0, import_import.importJWK)(__spreadProps(__spreadValues({}, jwk), { ext: true }), joseHeader.alg);
      if (keyObject instanceof Uint8Array || keyObject.type !== "public") {
        throw new import_errors.JWKSInvalid("JSON Web Key Set members must be public keys");
      }
      cached[joseHeader.alg] = keyObject;
    }
    return cached[joseHeader.alg];
  }
  async reload() {
    if (!this._pendingFetch) {
      this._pendingFetch = (0, import_fetch_jwks.default)(this._url, this._timeoutDuration, this._options).then((json) => {
        if (typeof json !== "object" || !json || !Array.isArray(json.keys) || !json.keys.every(isJWKLike)) {
          throw new import_errors.JWKSInvalid("JSON Web Key Set malformed");
        }
        this._jwks = { keys: json.keys };
        this._cooldownStarted = Date.now();
        this._pendingFetch = void 0;
      }).catch((err) => {
        this._pendingFetch = void 0;
        throw err;
      });
    }
    await this._pendingFetch;
  }
}
function createRemoteJWKSet(url, options) {
  return RemoteJWKSet.prototype.getKey.bind(new RemoteJWKSet(url, options));
}
module.exports = __toCommonJS(remote_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createRemoteJWKSet
});
