var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var decrypt_exports = {};
__export(decrypt_exports, {
  default: () => decrypt_default
});
var import_crypto = require("crypto");
var import_check_iv_length = __toESM(require("../lib/check_iv_length.js"));
var import_check_cek_length = __toESM(require("./check_cek_length.js"));
var import_buffer_utils = require("../lib/buffer_utils.js");
var import_errors = require("../util/errors.js");
var import_timing_safe_equal = __toESM(require("./timing_safe_equal.js"));
var import_cbc_tag = __toESM(require("./cbc_tag.js"));
var import_webcrypto = require("./webcrypto.js");
var import_crypto_key = require("../lib/crypto_key.js");
var import_is_key_object = __toESM(require("./is_key_object.js"));
var import_invalid_key_input = __toESM(require("../lib/invalid_key_input.js"));
var import_ciphers = __toESM(require("./ciphers.js"));
var import_is_key_like = require("./is_key_like.js");
function cbcDecrypt(enc, cek, ciphertext, iv, tag, aad) {
  const keySize = parseInt(enc.substr(1, 3), 10);
  if ((0, import_is_key_object.default)(cek)) {
    cek = cek.export();
  }
  const encKey = cek.subarray(keySize >> 3);
  const macKey = cek.subarray(0, keySize >> 3);
  const macSize = parseInt(enc.substr(-3), 10);
  const algorithm = `aes-${keySize}-cbc`;
  if (!(0, import_ciphers.default)(algorithm)) {
    throw new import_errors.JOSENotSupported(`alg ${enc} is not supported by your javascript runtime`);
  }
  const expectedTag = (0, import_cbc_tag.default)(aad, iv, ciphertext, macSize, macKey, keySize);
  let macCheckPassed;
  try {
    macCheckPassed = (0, import_timing_safe_equal.default)(tag, expectedTag);
  } catch {
  }
  if (!macCheckPassed) {
    throw new import_errors.JWEDecryptionFailed();
  }
  let plaintext;
  try {
    const decipher = (0, import_crypto.createDecipheriv)(algorithm, encKey, iv);
    plaintext = (0, import_buffer_utils.concat)(decipher.update(ciphertext), decipher.final());
  } catch {
  }
  if (!plaintext) {
    throw new import_errors.JWEDecryptionFailed();
  }
  return plaintext;
}
function gcmDecrypt(enc, cek, ciphertext, iv, tag, aad) {
  const keySize = parseInt(enc.substr(1, 3), 10);
  const algorithm = `aes-${keySize}-gcm`;
  if (!(0, import_ciphers.default)(algorithm)) {
    throw new import_errors.JOSENotSupported(`alg ${enc} is not supported by your javascript runtime`);
  }
  try {
    const decipher = (0, import_crypto.createDecipheriv)(algorithm, cek, iv, { authTagLength: 16 });
    decipher.setAuthTag(tag);
    if (aad.byteLength) {
      decipher.setAAD(aad, { plaintextLength: ciphertext.length });
    }
    const plaintext = decipher.update(ciphertext);
    decipher.final();
    return plaintext;
  } catch {
    throw new import_errors.JWEDecryptionFailed();
  }
}
const decrypt = (enc, cek, ciphertext, iv, tag, aad) => {
  let key;
  if ((0, import_webcrypto.isCryptoKey)(cek)) {
    (0, import_crypto_key.checkEncCryptoKey)(cek, enc, "decrypt");
    key = import_crypto.KeyObject.from(cek);
  } else if (cek instanceof Uint8Array || (0, import_is_key_object.default)(cek)) {
    key = cek;
  } else {
    throw new TypeError((0, import_invalid_key_input.default)(cek, ...import_is_key_like.types, "Uint8Array"));
  }
  (0, import_check_cek_length.default)(enc, key);
  (0, import_check_iv_length.default)(enc, iv);
  switch (enc) {
    case "A128CBC-HS256":
    case "A192CBC-HS384":
    case "A256CBC-HS512":
      return cbcDecrypt(enc, key, ciphertext, iv, tag, aad);
    case "A128GCM":
    case "A192GCM":
    case "A256GCM":
      return gcmDecrypt(enc, key, ciphertext, iv, tag, aad);
    default:
      throw new import_errors.JOSENotSupported("Unsupported JWE Content Encryption Algorithm");
  }
};
var decrypt_default = decrypt;
module.exports = __toCommonJS(decrypt_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
