var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var verify_exports = {};
__export(verify_exports, {
  flattenedVerify: () => flattenedVerify
});
var import_base64url = require("../../runtime/base64url.js");
var import_verify = __toESM(require("../../runtime/verify.js"));
var import_errors = require("../../util/errors.js");
var import_buffer_utils = require("../../lib/buffer_utils.js");
var import_is_disjoint = __toESM(require("../../lib/is_disjoint.js"));
var import_is_object = __toESM(require("../../lib/is_object.js"));
var import_check_key_type = __toESM(require("../../lib/check_key_type.js"));
var import_validate_crit = __toESM(require("../../lib/validate_crit.js"));
var import_validate_algorithms = __toESM(require("../../lib/validate_algorithms.js"));
async function flattenedVerify(jws, key, options) {
  if (!(0, import_is_object.default)(jws)) {
    throw new import_errors.JWSInvalid("Flattened JWS must be an object");
  }
  if (jws.protected === void 0 && jws.header === void 0) {
    throw new import_errors.JWSInvalid('Flattened JWS must have either of the "protected" or "header" members');
  }
  if (jws.protected !== void 0 && typeof jws.protected !== "string") {
    throw new import_errors.JWSInvalid("JWS Protected Header incorrect type");
  }
  if (jws.payload === void 0) {
    throw new import_errors.JWSInvalid("JWS Payload missing");
  }
  if (typeof jws.signature !== "string") {
    throw new import_errors.JWSInvalid("JWS Signature missing or incorrect type");
  }
  if (jws.header !== void 0 && !(0, import_is_object.default)(jws.header)) {
    throw new import_errors.JWSInvalid("JWS Unprotected Header incorrect type");
  }
  let parsedProt = {};
  if (jws.protected) {
    const protectedHeader = (0, import_base64url.decode)(jws.protected);
    try {
      parsedProt = JSON.parse(import_buffer_utils.decoder.decode(protectedHeader));
    } catch {
      throw new import_errors.JWSInvalid("JWS Protected Header is invalid");
    }
  }
  if (!(0, import_is_disjoint.default)(parsedProt, jws.header)) {
    throw new import_errors.JWSInvalid("JWS Protected and JWS Unprotected Header Parameter names must be disjoint");
  }
  const joseHeader = {
    ...parsedProt,
    ...jws.header
  };
  const extensions = (0, import_validate_crit.default)(import_errors.JWSInvalid, /* @__PURE__ */ new Map([["b64", true]]), options?.crit, parsedProt, joseHeader);
  let b64 = true;
  if (extensions.has("b64")) {
    b64 = parsedProt.b64;
    if (typeof b64 !== "boolean") {
      throw new import_errors.JWSInvalid('The "b64" (base64url-encode payload) Header Parameter must be a boolean');
    }
  }
  const { alg } = joseHeader;
  if (typeof alg !== "string" || !alg) {
    throw new import_errors.JWSInvalid('JWS "alg" (Algorithm) Header Parameter missing or invalid');
  }
  const algorithms = options && (0, import_validate_algorithms.default)("algorithms", options.algorithms);
  if (algorithms && !algorithms.has(alg)) {
    throw new import_errors.JOSEAlgNotAllowed('"alg" (Algorithm) Header Parameter not allowed');
  }
  if (b64) {
    if (typeof jws.payload !== "string") {
      throw new import_errors.JWSInvalid("JWS Payload must be a string");
    }
  } else if (typeof jws.payload !== "string" && !(jws.payload instanceof Uint8Array)) {
    throw new import_errors.JWSInvalid("JWS Payload must be a string or an Uint8Array instance");
  }
  let resolvedKey = false;
  if (typeof key === "function") {
    key = await key(parsedProt, jws);
    resolvedKey = true;
  }
  (0, import_check_key_type.default)(alg, key, "verify");
  const data = (0, import_buffer_utils.concat)(import_buffer_utils.encoder.encode(jws.protected ?? ""), import_buffer_utils.encoder.encode("."), typeof jws.payload === "string" ? import_buffer_utils.encoder.encode(jws.payload) : jws.payload);
  const signature = (0, import_base64url.decode)(jws.signature);
  const verified = await (0, import_verify.default)(alg, key, signature, data);
  if (!verified) {
    throw new import_errors.JWSSignatureVerificationFailed();
  }
  let payload;
  if (b64) {
    payload = (0, import_base64url.decode)(jws.payload);
  } else if (typeof jws.payload === "string") {
    payload = import_buffer_utils.encoder.encode(jws.payload);
  } else {
    payload = jws.payload;
  }
  const result = { payload };
  if (jws.protected !== void 0) {
    result.protectedHeader = parsedProt;
  }
  if (jws.header !== void 0) {
    result.unprotectedHeader = jws.header;
  }
  if (resolvedKey) {
    return { ...result, key };
  }
  return result;
}
module.exports = __toCommonJS(verify_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  flattenedVerify
});
