var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, copyDefault, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && (copyDefault || key !== "default"))
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toESM = (module2, isNodeMode) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", !isNodeMode && module2 && module2.__esModule ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
var __toCommonJS = /* @__PURE__ */ ((cache) => {
  return (module2, temp) => {
    return cache && cache.get(module2) || (temp = __reExport(__markAsModule({}), module2, 1), cache && cache.set(module2, temp), temp);
  };
})(typeof WeakMap !== "undefined" ? /* @__PURE__ */ new WeakMap() : 0);
var encrypt_exports = {};
__export(encrypt_exports, {
  default: () => encrypt_default
});
var import_buffer_utils = require("../lib/buffer_utils.js");
var import_check_iv_length = __toESM(require("../lib/check_iv_length.js"));
var import_check_cek_length = __toESM(require("./check_cek_length.js"));
var import_webcrypto = __toESM(require("./webcrypto.js"));
var import_crypto_key = require("../lib/crypto_key.js");
var import_invalid_key_input = __toESM(require("../lib/invalid_key_input.js"));
var import_errors = require("../util/errors.js");
var import_is_key_like = require("./is_key_like.js");
async function cbcEncrypt(enc, plaintext, cek, iv, aad) {
  if (!(cek instanceof Uint8Array)) {
    throw new TypeError((0, import_invalid_key_input.default)(cek, "Uint8Array"));
  }
  const keySize = parseInt(enc.substr(1, 3), 10);
  const encKey = await import_webcrypto.default.subtle.importKey("raw", cek.subarray(keySize >> 3), "AES-CBC", false, ["encrypt"]);
  const macKey = await import_webcrypto.default.subtle.importKey("raw", cek.subarray(0, keySize >> 3), {
    hash: `SHA-${keySize << 1}`,
    name: "HMAC"
  }, false, ["sign"]);
  const ciphertext = new Uint8Array(await import_webcrypto.default.subtle.encrypt({
    iv,
    name: "AES-CBC"
  }, encKey, plaintext));
  const macData = (0, import_buffer_utils.concat)(aad, iv, ciphertext, (0, import_buffer_utils.uint64be)(aad.length << 3));
  const tag = new Uint8Array((await import_webcrypto.default.subtle.sign("HMAC", macKey, macData)).slice(0, keySize >> 3));
  return { ciphertext, tag };
}
async function gcmEncrypt(enc, plaintext, cek, iv, aad) {
  let encKey;
  if (cek instanceof Uint8Array) {
    encKey = await import_webcrypto.default.subtle.importKey("raw", cek, "AES-GCM", false, ["encrypt"]);
  } else {
    (0, import_crypto_key.checkEncCryptoKey)(cek, enc, "encrypt");
    encKey = cek;
  }
  const encrypted = new Uint8Array(await import_webcrypto.default.subtle.encrypt({
    additionalData: aad,
    iv,
    name: "AES-GCM",
    tagLength: 128
  }, encKey, plaintext));
  const tag = encrypted.slice(-16);
  const ciphertext = encrypted.slice(0, -16);
  return { ciphertext, tag };
}
const encrypt = async (enc, plaintext, cek, iv, aad) => {
  if (!(0, import_webcrypto.isCryptoKey)(cek) && !(cek instanceof Uint8Array)) {
    throw new TypeError((0, import_invalid_key_input.default)(cek, ...import_is_key_like.types, "Uint8Array"));
  }
  (0, import_check_iv_length.default)(enc, iv);
  switch (enc) {
    case "A128CBC-HS256":
    case "A192CBC-HS384":
    case "A256CBC-HS512":
      if (cek instanceof Uint8Array)
        (0, import_check_cek_length.default)(cek, parseInt(enc.substr(-3), 10));
      return cbcEncrypt(enc, plaintext, cek, iv, aad);
    case "A128GCM":
    case "A192GCM":
    case "A256GCM":
      if (cek instanceof Uint8Array)
        (0, import_check_cek_length.default)(cek, parseInt(enc.substr(1, 3), 10));
      return gcmEncrypt(enc, plaintext, cek, iv, aad);
    default:
      throw new import_errors.JOSENotSupported("Unsupported JWE Content Encryption Algorithm");
  }
};
var encrypt_default = encrypt;
module.exports = __toCommonJS(encrypt_exports);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
